/*
 * Created on Mar 24, 2005
 *
 */
package it.dtt.ca;

import it.dtt.ca.exception.SessionClosedException;

/**
 * All CA related classes shall extend this class in order to be able to ensure
 * that the actual CA session is opened.
 * The CASession and CAObject mechanism works as follows: 
 * <ol>
 * <li>when a new CAManager object is created, it calls the empty constructor of
 * its parent class CAObject, which creates a new CASession object;</li>
 * <li>instead, when creating other objects derived from CAObject (Pin, Slot,
 * SmartCard), these objects share the same CASession instance (meaning: the same
 * session) of the parent object (typically: CAManager);</li>
 * <li>if the session - for any reason - is closed, all objects derived from
 * CAObject are "closed" and will reply with a SessionClosedException exception to
 * any method which includes a call to CAObject.checksession() </li></ol>
 */
public abstract class CAObject
{
    private CASession mySession = null;
    private CAObject parent = null;
   
    /**
     * Constructs the object using a new CA session.
     */
    protected CAObject()
    {
        super();
        mySession = new CASession(this);      
    }

    /**
     * Constructs the object using the same CA session of the parent object.
     *
     * @param parent the parent of this CA object.
     */
    protected CAObject(CAObject parent)
    {
        super();
        mySession = parent.getSession();
        this.parent = parent;
    }


    /**
     * Closes the session related with this CA object only if the current
     * session was created by this object, i.e. {@link CAObject#CAObject()}
     * was used to create this object.
     */
    protected final void closeSession()
    {
    }

    /**
     * This method is invoked before session closing.
     */
    protected abstract void sessionClosing();

    /**
     * This method is invoked after the session has been closed.
     */
    protected abstract void sessionClosed();

    /**
     * Checks if the actual session is opened. Does nothing if it is opened;
     * throws an exception otherwise. It shall be used at the beginning
     * of every method which throws {@link SessionClosedException} exception.
     *
     * @throws SessionClosedException thrown if the actual session is closed.
     */
    protected final void checkSession() throws SessionClosedException
    {
        if (! mySession.isSessionOpen())
            throw new SessionClosedException();
    }
    
    /**
     * Get the session associated with this CAObject.
     *
     * @return the session.
     */
    public CASession getSession()
    {
        return mySession;
    }
  
    /**
     * Return the parent object of this CAObject.
     * @return the parent CAObject.
     */
    public CAObject getParent()
    {
        return parent;
    }
}
