/*
 * Created on Mar 29, 2005
 */
package it.dtt.ca;

import it.dtt.ca.exception.*;


/**
 * This class represents a Personal Identification Number (PIN).
 * It allows to use and change the PIN codes stored either on the STB or on the smart card.
 * 
 */
public abstract class Pin extends CAObject {

    /**
     * Indicates the PIN protecting the CA System functionalities of the smart card.
     */
    public final static String SMART_CARD_PIN = "SMART CARD";

    /**
     * Indicates the PIN proteting access to history data.
     * This functionality allows the user to read the history of previous purchases, recharges, messages from the CAS, etc.
     */
    public final static String HISTORY_PIN = "HISTORY";

    /**
     * Indicates the PIN providing parental control.
     * This functionality allows the user to block content which is not suitable for children.
     */
    public final static String PARENTAL_CONTROL_PIN = "PARENTAL CONTROL";

    /**
     * Create an instance of a PIN.
     *
     * @param owner the owner of this PIN, either a SmartCard or a CAManager.
     * @param name the name of this PIN.
     */
    protected Pin(CAObject owner, String name) {
    }

    /**
     * Get the name that identifies this PIN.
     *
     * @return the name of this PIN.
     */
    public String getPinName() {
        return null;
    }

    /**
     * Enable or disable the PIN code usage.
     *
     * @param required pin mode, when true the PIN is required.
     * @throws CardNotFoundException  if the PIN was stored on a smart card that is not present anymore in the slot.
     * @throws SessionClosedException if the session is closed.
     * @throws AccessDeniedException  if enabling or disabling the PIN code is not allowed.
     */
    public abstract void setRequired(boolean required) throws CardNotFoundException, SessionClosedException, AccessDeniedException;

    /**
     * This method is used to determine if the specific PIN is required.
     * If this is the case, the PIN must be inserted before performing any operation under control of this PIN.
     * @return true if the pin is required, false otherwise.
     * @throws CardNotFoundException  if the PIN was stored on a smart card that is not present anymore in the slot.
     * @throws SessionClosedException if the session is closed.
     */
    public abstract boolean isRequired() throws CardNotFoundException, SessionClosedException;

    /**
     * Check if the Pin is correct.
     * 
     * The Pin must be provided in BCD format. 
     * e.g.: A Pin of 1234 will be provided as codebyte[0] = 0x12 and codebyte[1] = 0x34.
     * 
     * @param pin  the pin entered.
     * @return true if pin is correct.
     *
     * @throws CardNotFoundException  if the PIN was stored on a smart card that is not present anymore in the slot.
     * @throws SessionClosedException if the session is closed.
     */

    public abstract boolean check(byte[] pin) throws CardNotFoundException, SessionClosedException; 

    /**
     * Retrieves remaining tries of the pin before smart card is blocked.
     * @return number of tries remaining.
     * @throws CardNotFoundException if the PIN was stored on a smart card that is not present anymore in the slot.
     * @throws SessionClosedException if the session is closed.
     */
    public abstract byte getTriesRemaining() throws CardNotFoundException, SessionClosedException;

    /**
     * Retrieves if the current pin is validated since the smart card has been reset last time
     * @return true if pin is validated
     * @throws CardNotFoundException if the PIN was stored on a smart card that is not present anymore in the slot.
     * @throws SessionClosedException if the session is closed.
     */
    public abstract boolean isValidated() throws CardNotFoundException, SessionClosedException;

    /**
     * Resets the validate flag. This means that the pin will be required again next time.
     * @throws CardNotFoundException if the PIN was stored on a smart card that is not present anymore in the slot.
     * @throws SessionClosedException if the session is closed.
      */
    public abstract void reset() throws CardNotFoundException, SessionClosedException;

    /**
     * This method sets a new value for the PIN, resets the PIN try counter to the value of the PIN try limit, and
     * resets the validated flag.
     *
     * The Pin must be provided in BCD format. 
     * e.g.: A Pin of 1234 will be provided as codebyte[0] = 0x12 and codebyte[1] = 0x34.
     * 
     * @param oldPin the byte array containing the old PIN value.
     * @param newPin the byte array containing the new PIN value.
     * @throws InvalidPinException if the oldPin is not correct, or if newPin length is greater than the maximum PIN size.
     * @throws CardNotFoundException if the PIN was stored on a smart card that is not present anymore in the slot.
     * @throws SessionClosedException if the session is closed.
     * @throws AccessDeniedException if changing the PIN code is not allowed.
     */
    public abstract void change(byte[] oldPin, byte[] newPin) throws InvalidPinException, CardNotFoundException, SessionClosedException, AccessDeniedException;
    
    /**
     * This method sets a new value for the PIN, resets the PIN try counter to the value of the PIN try limit, and
     * resets the validated flag.
     * 
     * The Pin must be provided in BCD format. 
     * e.g.: A Pin of 1234 will be provided as codebyte[0] = 0x12 and codebyte[1] = 0x34.
     * 
     *
     * @deprecated 
     * @param pin the byte array containing the new PIN value.
     * @throws InvalidPinException if the oldPin is not correct, or if newPin length is greater than the maximum PIN size.
     * @throws CardNotFoundException if the PIN was stored on a smart card that is not present anymore in the slot.
     * @throws SessionClosedException if the session is closed.
     * @throws AccessDeniedException if changing the PIN code is not allowed.
     */
    public abstract void change(byte[] pin) throws InvalidPinException, CardNotFoundException, SessionClosedException,AccessDeniedException;

}
