package it.dtt.ca;

import it.dtt.ca.event.SlotListener;
import it.dtt.ca.exception.AccessDeniedException;
import it.dtt.ca.exception.CardNotFoundException;
import it.dtt.ca.exception.SessionClosedException;

/**
 * This class represents the physical slot in which a smart card can be inserted.
 * Access to the slot via the class methods is allowed as long as the current session remains open, 
 * i.e. until the application calls it.dtt.ca.CAManagerFactory.closeSession() to close the current session; 
 * subsequently, the class methods will throw the it.dtt.ca.exception.SessionClosedException exception.  
 * This behaviour is implemented by CAObject from which Slot derives.
 */


public abstract class Slot extends CAObject {

    /**
     * Create a slot instance.
     * @param manager the CA Manager controlling this slot.
     * @param id the id of this slot.
     */
    protected Slot(CAManager manager, int id) {
    }

    /**
     * Returns the number identifying this slot.
     * The first slot is numbered zero; this is normally the only slot present in the STB.
     *
     * @return number identifying the slot.
     * @throws SessionClosedException if the session is closed.
     */
    public int getSlotId() throws SessionClosedException {
        return 0;
    }

    /**
     * A <code>SlotListener</code> is notified if the card is inserted or removed
     * from the slot.
     *
     * @param listener object listening to slot changes.
     * @throws SessionClosedException if the session is closed.
     */
    public void addSlotListener(SlotListener listener) throws SessionClosedException {
    }

    /**
     * Remove a previously added <code>SlotListener</code>.
     *
     * @param listener object to be removed.
     * @throws SessionClosedException if the session is closed.
     */
    public void removeSlotListener(SlotListener listener) throws SessionClosedException {
    }

    /**
     * Tells if a smart card is inserted in the slot.
     * The return value is one of the values defined in it.dtt.ca.event.SlotEvent.
     * See SlotEvent for a description of the semantics of each value.
     *
     * @return a SlotEvent costant value
     * @throws SessionClosedException if the session is closed.
     * @see it.dtt.ca.event.SlotEvent
     */
    public abstract int getStatus() throws SessionClosedException;

    /**
     * Get the Smart Card object associated with this card reader.
     *
     * @return smart card related object
     * @throws it.dtt.ca.exception.CardNotFoundException if a card is not inserted in the slot
     * @throws it.dtt.ca.exception.AccessDeniedException if the wrong card is inserted in the slot (i.e. a card which do not correspond to the CA kernel in use)
     * @throws SessionClosedException if the session is closed.
     * @see SmartCard
     * @see it.dtt.ca.exception.CardNotFoundException
     * @see it.dtt.ca.exception.AccessDeniedException
     * @see SessionClosedException
     */
    public abstract SmartCard getSmartCard() throws CardNotFoundException, AccessDeniedException, SessionClosedException;
}
