package it.dtt.ca;

import it.dtt.ca.event.*;
import it.dtt.ca.exception.*;
import it.dtt.ca.util.Wallet;

import java.util.*;

/**
 * This is the class used to retrieve information from the smart card currently inserted in one of the STB slots.
 * 
 */
public abstract class SmartCard extends CAObject {
    
    /**
     * Create an instance of a SmartCard.
     * @param slot the slot which owns this SmartCard.
     */
    protected SmartCard(Slot slot) {
        super(slot);
    }

    /**
     * Get all Personal Identification Numbers that are stored on the smart card.
     *
     * @return a vector of PinController objects.
     * @throws CardNotFoundException  if the smart card is removed or is not present in the slot.
     * @throws SessionClosedException if the session is closed.
     */
    public abstract Pin[] getPins() throws CardNotFoundException, SessionClosedException;

    /**
     * Get the array of wallets provided by this smart card.
     *
     * @return the array of Wallets.
     * @throws CardNotFoundException  if the smart card is removed or is not present in the slot.
     * @throws SessionClosedException if the session is closed.
     */
    public abstract Wallet[] getWallets() throws CardNotFoundException, SessionClosedException;

    /**
     * Set the wallet currently in use.
     *
     * @param wallet the wallet to use from now on.
     * @throws CardNotFoundException       if the smart card is removed or is not present in the slot.
     * @throws SessionClosedException      if the session is closed.
     * @throws AccessDeniedException       if access is denied for security reasons.
     * @throws UnsupportedFeatureException if the smart card does not support setting a different wallet than the default one.
     */
    public void setUsedWallet(Wallet wallet) throws CardNotFoundException, SessionClosedException, AccessDeniedException, UnsupportedFeatureException {
    }

    /**
     * Get the wallet currently in use.
     *
     * @return the wallet currently in use.
     * @throws CardNotFoundException  if the smart card is removed or is not present in the slot.
     * @throws SessionClosedException if the session is closed.
     */
    public Wallet getUsedWallet() throws CardNotFoundException, SessionClosedException {
        return null;
    }

    /**
     * Gets the smart card identification number for the given card.
     *
     * @return the smart card id.
     * @throws CardNotFoundException  if the smart card is removed or is not present in the slot.
     * @throws SessionClosedException if the session is closed.
     */
    public abstract String getNumber() throws CardNotFoundException, SessionClosedException;

    /**
     * Gets the smart card version for the given card.
     *
     * @return the smart card version.
     * @throws CardNotFoundException  if the smart card is removed or is not present in the slot.
     * @throws SessionClosedException if the session is closed.
     */
    public abstract String getVersion() throws CardNotFoundException, SessionClosedException;

    /**
     * Get the expiry date of this smart card.
     *
     * @return the expiry date of this smart card.
     * @throws CardNotFoundException  if the smart card is removed or is not present in the slot.
     * @throws SessionClosedException if the session is closed.
     */
    public abstract Date getExpiryDate() throws CardNotFoundException, SessionClosedException;

    /**
     * Retrieves the name of the service provider who delivered the card.
     *
     * @return name of the broadcaster providing the service.
     * @throws CardNotFoundException  if the smart card is removed or is not present in the slot.
     * @throws SessionClosedException if the session is closed.
     */
    public abstract String getServiceProviderName() throws CardNotFoundException, SessionClosedException;

    /**
     * Retrieves more CA related properties that cannot be retrieved with other
     * methods (such as revisions or sw details).
     *
     * @return a list of properties
     * @throws CardNotFoundException  if the smart card is removed or is not present in the slot.
     * @throws SessionClosedException if the session is closed.
     */
    public abstract Properties getMoreInfo() throws CardNotFoundException, SessionClosedException;

    /**
     * Retrieves the byte array representing the Answer-to-Reset.
     *
     * @return the smart card Answer-to-Reset
     * @throws it.dtt.ca.exception.UnsupportedFeatureException
     *                                if the CAS does not support ATR retrieval
     * @throws CardNotFoundException  if the smart card is removed or is not present in the slot.
     * @throws SessionClosedException if the session is closed.
     */
    public abstract byte[] getATR() throws UnsupportedFeatureException, CardNotFoundException, SessionClosedException;
    
    /**
     * A <code>SmartCardListener</code> is notified if any change occur in the smart card.
     *
     * @param listener object listening to smart card changes.
     * @throws SessionClosedException if the session is closed.
     */
    public void addSmartCardListener(SmartCardListener listener) throws SessionClosedException {
    }

    /**
     * Remove a previously added <code>SmartCardListener</code>.
     *
     * @param listener object to be removed.
     * @throws SessionClosedException if the session is closed.
     */
    public void removeSmartCardListener(SmartCardListener listener) throws SessionClosedException {
    }

    /**
     * Return the smart card status. The return value is one of the values specified in it.dtt.ca.event.SmartCardStatusEvent.
     *
     * @return an int representing a SmartCardEvent costant value.
     * @throws CardNotFoundException  if the smart card is removed or is not present in the slot.
     * @throws SessionClosedException if the session is closed.
     * @see it.dtt.ca.event.SmartCardEvent
     */
    public abstract int getStatus() throws CardNotFoundException, SessionClosedException;

    /**
     * Read the User Data field from the smart card.
     *
     * @return array of bytes stored in the smart card as user data.
     * @throws it.dtt.ca.exception.UnsupportedFeatureException
     *                                if the Smart Card does not support user data retrieval
     * @throws CardNotFoundException  if the smart card is removed or is not present in the slot.
     * @throws SessionClosedException if the session is closed.
     */
    public byte[] getUserData() throws UnsupportedFeatureException, CardNotFoundException, SessionClosedException {
        return null;
    }

    /**
     * Write the User Data field on the smart card.
     *
     * @param userData the data to write in the user data field of the smart card
     * @return true if the write operation on the smart card is successful, false otherwise.
     * @throws UnsupportedFeatureException if the Smart Card does not support user data storage
     * @throws CardNotFoundException       if the smart card is removed or is not present in the slot.
     * @throws SessionClosedException      if the session is closed.
     */
    public boolean setUserData(byte[] userData) throws UnsupportedFeatureException, CardNotFoundException, SessionClosedException {
        return true;
    }
    
    /**
     * Get the name of the smart card provider (normally the same as the CA provider name).
     *
     * @return a string containing the name of the smart card provider.
     * @throws CardNotFoundException       if the smart card is removed or is not present in the slot.
     * @throws SessionClosedException      if the session is closed.
     */
    public abstract String getProvider() throws CardNotFoundException, SessionClosedException;

}