package it.dtt.ca.event;

import it.dtt.ca.*;

/**
 * This event notifies user about changes in the slot status (e.g.
 * if a smart card is present or not in the given slot). It notifies also about card insertion
 * errors or if the box cannot communicate with the smart card for any reason.
 * 
 */
public class SlotEvent extends CAEvent {

    /**
     * Notifies a card insertion;
     * this value is also returned when a card is present in the slot and a slot status check is performed.
     *
     * @see Slot
     */
    public final static int CARD_IN = 350;
    /**
     * Notifies a card removal;
     * this value is also returned when a card is not present in the slot and a slot status check is performed.
     *
     * @see Slot
     */
    public final static int CARD_OUT = 351;
    /**
     * Notifies a smart card is inserted into the reader but wrong ATR is
     * received (e.g. because of a damaged card").
     * 
     * @see Slot
     */
    public final static int CARD_ERROR = 352;
    /**
     * Notifies that smart card is inserted into the reader but no ATR is retrieved
	 * because no electrical communication is established with the smart card (e.g.
	 * card upside-down).
	 * 
     * @see Slot
     */
    public final static int CARD_MUTED = 353;
    /**
     * Notifies that access to the card currently inserted in the slot is denied;
     * this normally means that the card does not correspond to the current active service and CAS.
     * 
     */
    public final static int CARD_ACCESS_DENIED = 354;

    /**
     * Notifies that the smart card is being verified
     */
    public final static int CARD_VERIFYING = 355;

    /**
     * Notifies that status of the slot has not been retrieved yet
     */
    public final static int ERROR_UNKNOWN = 356;


    /**
     * Create a SlotEvent instance
     *
     * @param slot the slot generating the event
     */
    protected SlotEvent(it.dtt.ca.Slot slot, String description) {
        super(slot, null, description);
    }

    /**
     * Gives the slot which generated the event.
     * 
     * @return the interested slot
     */
    public Slot getSlot() {
        return (Slot) getSource();  
    }

    /**
     * Returns the type of event that's arrived.
     *
     * @return type of event.
     * @see it.dtt.ca.event.CAEvent#SLOT_EVENT
     */
    public int getType() {
        return SLOT_EVENT;
    }

}