/*
 * Created on Jan 2, 2005
 */
package it.dtt.ca.ppv;

import it.dtt.ca.util.Money;
import it.dtt.ca.util.Duration;
import it.dtt.ca.exception.FieldNotPresentException;
import it.dtt.ca.CAPrivateData;

import java.util.Date;

/**
 * Abstract class representing content that can be purchased through the CA system. It is the common base class for packages of events, single PPV events, and subscription services.
 * 
 * Where methods take an array as an input parameter,
 * implementations should take a copy of the array. Where methods return an array,
 * a copy should be returned and modifications to the array should be ignored by
 * the implementation until or unless the array is passed back to the
 * implementation as a parameter to a method call.
 */
public abstract class Product {

    public static final int GENERIC_PRODUCT = 2001;
    public static final int PPV_EVENT = 2002;
    public static final int PPT_EVENT = 2003;
    public static final int SUBSCRIPTION = 2004;
    public static final int PPV_PACKAGE = 2005;

    /**
     * Called by derived classes to create a product.
     *
     * @param id   the product id.
     * @param name the product name.
	 * @param data the private data associated with this product.
	 * @param description the description for this product.
	 * @param extendedDescription the extended description for this product.
	 * @param pwStartTime the time the purchase window starts.
	 * @param pwEndTime the time the purchase window ends.
	 * @param price the price to purchase this product.
	 * @param preview the preview time for this product.
     */
    protected Product(String id, String name, CAPrivateData data, String[] description, String[] extendedDescription, Date pwStartTime, Date pwEndTime, Money price, Duration preview) {
    }

    /**
     * Called by derived classes to create a product.
     *
     * @param id   the product id.
     * @param name the product name.
     */
	
    protected Product(String id, String name, CAPrivateData privateData) {
    }

    /**
	 * Check if two products have the same CAS ID, meaning they are the same exact product
	 * 
	 * @param obj the object that is compared with current one
	 * @return true if the CAS ID of the two products is the same
	 */

	public boolean equals(Object obj) {
		return true;
	}

	/**
	 * Returns hash code for the given instance. If equals() method return true for two products, than calling 
	 * this method on the two products must return the same value. 
	 * 
	 * @return the hash code associated to the instance.
	 */

	public int hashCode() {
		return 0;
	}



    /**
     * Returns a CAS dependent ID for the specific product.
     *
     * @return the id of this product.
     */
    public String getId() {
        return null;
    }

    /**
     * Returns Private data for the specific product.
     *
     * @return the Private data of this product, or null if none exists.
     */
    public CAPrivateData getPrivateData() {
        return null;
    }

    /**
	 * @deprecated no replacement
	 *
     * Sets the Private Data for this event.
     */
    protected void setPrivateData(CAPrivateData privateData) {
    }

    /**
     * Returns the product type
     * @return product type
     * @see  it.dtt.ca.ppv.Product#GENERIC_PRODUCT
     * @see  it.dtt.ca.ppv.Product#PPV_EVENT
     * @see  it.dtt.ca.ppv.Product#SUBSCRIPTION
     * @see  it.dtt.ca.ppv.Product#PPV_PACKAGE
     */
    public abstract int getType();

    /**
     * Returns the product name.
     *
     * @return the name
     */
    public String getName() {
        return null;
    }

    /**
     * Returns the product description.
     *
     * @return the description.
     * @throws FieldNotPresentException if the description is not available.
     */
    public String[] getDescription() throws FieldNotPresentException
    {
        return null;
    }
    
    /**
	 * @deprecated no replacement
	 *
     * Set the product description.
     * @param description The description to set.
     */
    protected void setDescription(String[] description) {
    }
    
    /**
     * Returns the product extended description (e.g. conditions of purchase).
     *
     * @return the extended description.
     * @throws FieldNotPresentException if the field is not stored
     */
    public String[] getExtendedDescription() throws FieldNotPresentException {
        return null;
    }
    
    /**
	 * @deprecated no replacement
	 *
     * Set the extended description of this Product.
     * @param extendedDescription The extendedDescription to set.
     */
    protected void setExtendedDescription(String[] extendedDescription) {
    }

    /**
     * Get the starting time of the purchase window for this product.
     *
     * @return the date and time from which purchasing is possible.
     * @throws FieldNotPresentException if the field is not stored.
     */
    public Date getPurchaseWindowStartTime() throws FieldNotPresentException
    {
        return null;
    }

    /**
	 * @deprecated no replacement
	 *
     * Set the starting time of the purchase window for this product.
     * @param purchaseWindowStartTime The purchaseWindowStartTime to set.
     */
    protected void setPurchaseWindowStartTime(Date purchaseWindowStartTime) {
    }
    
    /**
     * Get the ending time of the purchase window for this product.
     *
     * @return the date and time from which purchasing is not possible anymore.
     * @throws it.dtt.ca.exception.FieldNotPresentException
     *          if the field is not stored
     */
    public Date getPurchaseWindowEndTime() throws FieldNotPresentException {
        return null;
    }
   
    /**
	 * @deprecated no replacement
	 *
     * Set the ending time of the purchase window for this product.
     * @param purchaseWindowEndTime The purchaseWindowEndTime to set.
     */
    protected void setPurchaseWindowEndTime(Date purchaseWindowEndTime) {
    }
    
    /**
     * Get other products contained in this product.
     * This method can be used to offer addtional products related to the
     * current product to the end user.
     * <p>i.e. We may offer the end-user the chance to buy the directors commentary for a movie.
     * <p>Or in the case of a PPV Package it will contain all the products in the PPV Package.
     *
     * @return Returns the contained products.
     */
    public Product[] getContainedProducts() {
        return null;
    }

    /**
     * Get the price of the product.
     * In case of a package, this may be different from the sum of the price of the individual events.
     *
     * @return the price of the product.
     * @throws FieldNotPresentException if the field is not stored
     */
    public Money getPrice() throws FieldNotPresentException
    {
        return null;
    }

    /**
	 * @deprecated no replacement
	 *
     * Set the price of this Product.
     * @param price The price to set.
     */
    void setPrice(Money price) {
    }
    
    /**
     * Check if the product is freely viewable.
     *
     * @return true if the product is free, false if the product is a pay per view product.
     * @throws it.dtt.ca.exception.FieldNotPresentException
     *          if the field is not stored
     */
    public abstract boolean isFree() throws FieldNotPresentException;

    /**
     * Get the total preview time for this product.
     * If the Product is a PPVPackage instance, this time represents the total free preview time which can be spent on any product contained in the package. 
     * This method will return a Duration of zero if the product has no free preview time, or if the product is free.
     *
     * @return the free preview time as a Duration instance.
     * @throws FieldNotPresentException if the preview time is not available.
     */
    public Duration getPreviewTime() throws FieldNotPresentException
    {
        return null;
    }

    /**
	 * @deprecated no replacement
	 *
     * Set the total preview time for this product.
     * @param previewTime The previewTime to set.
     */
    protected void setPreviewTime(Duration previewTime) {
    }
}
